@@ -139,24 +139,24 @@ static int channelscan_execute_callback(channelscan_callback_t callback, va_list
 static int channelscan_execute_internal(struct hdhomerun_device_t *hd, uint32_t channel_map, struct hdhomerun_channel_entry_t **pentry, channelscan_callback_t callback, va_list callback_ap)
 {
 	struct hdhomerun_channel_entry_t *entry = *pentry;
-	uint32_t frequency = entry->frequency;
+	uint32_t frequency = hdhomerun_channel_entry_frequency(entry);
 	char buffer[256];
 	int ret;
 
 	/* Combine channels with same frequency. */
 	char *ptr = buffer;
-	char *limit = buffer + sizeof(buffer);
 	sprintf(ptr, "%ld (", (unsigned long)frequency);
 	ptr = strchr(ptr, 0);
 	while (1) {
-		hdhomerun_channel_name(ptr, limit, entry);
+		const char *name = hdhomerun_channel_entry_name(entry);
+		strcpy(ptr, name);
 		ptr = strchr(ptr, 0);
 
 		entry = hdhomerun_channel_list_next(channel_map, entry);
 		if (!entry) {
 			break;
 		}
-		if (entry->frequency != frequency) {
+		if (hdhomerun_channel_entry_frequency(entry) != frequency) {
 			break;
 		}
 
