@@ -47,6 +47,7 @@ extern uint32_t hdhomerun_control_get_device_id(struct hdhomerun_control_sock_t
 extern uint32_t hdhomerun_control_get_device_ip(struct hdhomerun_control_sock_t *cs);
 extern uint32_t hdhomerun_control_get_device_id_requested(struct hdhomerun_control_sock_t *cs);
 extern uint32_t hdhomerun_control_get_device_ip_requested(struct hdhomerun_control_sock_t *cs);
+
 extern void hdhomerun_control_set_device(struct hdhomerun_control_sock_t *cs, uint32_t device_id, uint32_t device_ip);
 
 /*
@@ -59,9 +60,9 @@ extern void hdhomerun_control_set_device(struct hdhomerun_control_sock_t *cs, ui
 extern uint32_t hdhomerun_control_get_local_addr(struct hdhomerun_control_sock_t *cs);
 
 /*
- * Get the low-level socket handle.
+ * Low-level communication.
  */
-extern int hdhomerun_control_get_sock(struct hdhomerun_control_sock_t *cs);
+extern int hdhomerun_control_send_recv(struct hdhomerun_control_sock_t *cs, struct hdhomerun_pkt_t *tx_pkt, struct hdhomerun_pkt_t *rx_pkt, uint16_t type);
 
 /*
  * Get/set a control variable on the device.
@@ -94,6 +95,11 @@ extern int hdhomerun_control_set(struct hdhomerun_control_sock_t *cs, const char
  */
 extern int hdhomerun_control_upgrade(struct hdhomerun_control_sock_t *cs, FILE *upgrade_file);
 
+/*
+ * Debug logging.
+ */
+extern void hdhomerun_control_set_debug(struct hdhomerun_control_sock_t *cs, struct hdhomerun_debug_t *dbg);
+
 #ifdef __cplusplus
 }
 #endif
