@@ -47,15 +47,21 @@ struct hdhomerun_debug_t
 	pthread_t thread;
 	volatile bool_t enabled;
 	volatile bool_t terminate;
+	char *prefix;
 
+	pthread_mutex_t print_lock;
 	pthread_mutex_t queue_lock;
+	pthread_mutex_t send_lock;
+
 	struct hdhomerun_debug_message_t *queue_head;
 	struct hdhomerun_debug_message_t *queue_tail;
 	uint32_t queue_depth;
 
-	pthread_mutex_t sock_lock;
-	int sock;
 	uint64_t connect_delay;
+
+	char *file_name;
+	FILE *file_fp;
+	int sock;
 };
 
 static THREAD_FUNC_PREFIX hdhomerun_debug_thread_execute(void *arg);
@@ -69,8 +75,9 @@ struct hdhomerun_debug_t *hdhomerun_debug_create(void)
 
 	dbg->sock = -1;
 
+	pthread_mutex_init(&dbg->print_lock, NULL);
 	pthread_mutex_init(&dbg->queue_lock, NULL);
-	pthread_mutex_init(&dbg->sock_lock, NULL);
+	pthread_mutex_init(&dbg->send_lock, NULL);
 
 	if (pthread_create(&dbg->thread, NULL, &hdhomerun_debug_thread_execute, dbg) != 0) {
 		free(dbg);
@@ -80,6 +87,18 @@ struct hdhomerun_debug_t *hdhomerun_debug_create(void)
 	return dbg;
 }
 
+/* Send lock held by caller */
+static void hdhomerun_debug_close_file(struct hdhomerun_debug_t *dbg)
+{
+	if (!dbg->file_fp) {
+		return;
+	}
+
+	fclose(dbg->file_fp);
+	dbg->file_fp = NULL;
+}
+
+/* Send lock held by caller */
 static void hdhomerun_debug_close_sock(struct hdhomerun_debug_t *dbg)
 {
 	if (dbg->sock == -1) {
@@ -95,24 +114,79 @@ void hdhomerun_debug_destroy(struct hdhomerun_debug_t *dbg)
 	dbg->terminate = TRUE;
 	pthread_join(dbg->thread, NULL);
 
+	hdhomerun_debug_close_file(dbg);
 	hdhomerun_debug_close_sock(dbg);
 
+	if (dbg->prefix) {
+		free(dbg->prefix);
+	}
+
 	free(dbg);
 }
 
+void hdhomerun_debug_set_prefix(struct hdhomerun_debug_t *dbg, const char *prefix)
+{
+	pthread_mutex_lock(&dbg->print_lock);
+
+	if (dbg->prefix) {
+		free(dbg->prefix);
+		dbg->prefix = NULL;
+	}
+
+	if (prefix) {
+		dbg->prefix = strdup(prefix);
+	}
+
+	pthread_mutex_unlock(&dbg->print_lock);
+}
+
+void hdhomerun_debug_set_filename(struct hdhomerun_debug_t *dbg, const char *filename)
+{
+	pthread_mutex_lock(&dbg->send_lock);
+
+	if (!filename && !dbg->file_name) {
+		pthread_mutex_unlock(&dbg->send_lock);
+		return;
+	}
+	if (filename && dbg->file_name) {
+		if (strcmp(filename, dbg->file_name) == 0) {
+			pthread_mutex_unlock(&dbg->send_lock);
+			return;
+		}
+	}
+
+	hdhomerun_debug_close_file(dbg);
+	hdhomerun_debug_close_sock(dbg);
+
+	if (dbg->file_name) {
+		free(dbg->file_name);
+		dbg->file_name = NULL;
+	}
+	if (filename) {
+		dbg->file_name = strdup(filename);
+	}
+
+	pthread_mutex_unlock(&dbg->send_lock);
+}
+
 void hdhomerun_debug_enable(struct hdhomerun_debug_t *dbg)
 {
+	pthread_mutex_lock(&dbg->send_lock);
+
 	dbg->enabled = TRUE;
+
+	pthread_mutex_unlock(&dbg->send_lock);
 }
 
 void hdhomerun_debug_disable(struct hdhomerun_debug_t *dbg)
 {
-	pthread_mutex_lock(&dbg->sock_lock);
+	pthread_mutex_lock(&dbg->send_lock);
 
 	dbg->enabled = FALSE;
+	hdhomerun_debug_close_file(dbg);
 	hdhomerun_debug_close_sock(dbg);
 
-	pthread_mutex_unlock(&dbg->sock_lock);
+	pthread_mutex_unlock(&dbg->send_lock);
 }
 
 bool_t hdhomerun_debug_enabled(struct hdhomerun_debug_t *dbg)
@@ -169,14 +243,22 @@ void hdhomerun_debug_vprintf(struct hdhomerun_debug_t *dbg, const char *fmt, va_
 
 	time_t t = time(NULL);
 	strftime(ptr, end - ptr, "%Y%m%d-%H:%M:%S ", localtime(&t));
+	ptr = strchr(ptr, 0);
+
+	pthread_mutex_lock(&dbg->print_lock);
+
+	if (dbg->prefix) {
+		snprintf(ptr, end - ptr, "%s ", dbg->prefix);
+		ptr = strchr(ptr, 0);
+	}
+
+	pthread_mutex_unlock(&dbg->print_lock);
 
-	ptr = strchr(ptr, '\0');
 	vsnprintf(ptr, end - ptr, fmt, args);
 
-	ptr = strchr(ptr, '\0') - 1;
+	ptr = strchr(ptr, 0) - 1;
 	if (*ptr++ != '\n') {
-		*ptr++ = '\n';
-		*ptr++ = 0;
+		strcpy(ptr, "\n");
 	}
 
 	pthread_mutex_lock(&dbg->queue_lock);
@@ -194,32 +276,46 @@ void hdhomerun_debug_vprintf(struct hdhomerun_debug_t *dbg, const char *fmt, va_
 	pthread_mutex_unlock(&dbg->queue_lock);
 }
 
+/* Send lock held by caller */
+static bool_t hdhomerun_debug_output_message_file(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
+{
+	if (!dbg->file_fp) {
+		uint64_t current_time = getcurrenttime();
+		if (current_time < dbg->connect_delay) {
+			return FALSE;
+		}
+		dbg->connect_delay = current_time + 60*1000;
+
+		dbg->file_fp = fopen(dbg->file_name, "a");
+		if (!dbg->file_fp) {
+			return FALSE;
+		}
+	}
+
+	fprintf(dbg->file_fp, "%s", message->buffer);
+	fflush(dbg->file_fp);
+
+	return TRUE;
+}
+
+/* Send lock held by caller */
 #if defined(__CYGWIN__)
-static bool_t hdhomerun_debug_output_message(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
+static bool_t hdhomerun_debug_output_message_sock(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
 {
 	return TRUE;
 }
 #else
-static bool_t hdhomerun_debug_output_message(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
+static bool_t hdhomerun_debug_output_message_sock(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
 {
-	pthread_mutex_lock(&dbg->sock_lock);
-
-	if (!dbg->enabled) {
-		pthread_mutex_unlock(&dbg->sock_lock);
-		return TRUE;
-	}
-
 	if (dbg->sock == -1) {
 		uint64_t current_time = getcurrenttime();
 		if (current_time < dbg->connect_delay) {
-			pthread_mutex_unlock(&dbg->sock_lock);
 			return FALSE;
 		}
 		dbg->connect_delay = current_time + 60*1000;
 
 		dbg->sock = (int)socket(AF_INET, SOCK_STREAM, 0);
 		if (dbg->sock == -1) {
-			pthread_mutex_unlock(&dbg->sock_lock);
 			return FALSE;
 		}
 
@@ -232,30 +328,46 @@ static bool_t hdhomerun_debug_output_message(struct hdhomerun_debug_t *dbg, stru
 		struct addrinfo *sock_info;
 		if (getaddrinfo(HDHOMERUN_DEBUG_HOST, HDHOMERUN_DEBUG_PORT, &hints, &sock_info) != 0) {
 			hdhomerun_debug_close_sock(dbg);
-			pthread_mutex_unlock(&dbg->sock_lock);
 			return FALSE;
 		}
 		if (connect(dbg->sock, sock_info->ai_addr, (int)sock_info->ai_addrlen) != 0) {
 			freeaddrinfo(sock_info);
 			hdhomerun_debug_close_sock(dbg);
-			pthread_mutex_unlock(&dbg->sock_lock);
 			return FALSE;
 		}
 		freeaddrinfo(sock_info);
 	}
 
-	size_t length = strlen(message->buffer);	
+	size_t length = strlen(message->buffer);
 	if (send(dbg->sock, (char *)message->buffer, (int)length, 0) != length) {
 		hdhomerun_debug_close_sock(dbg);
-		pthread_mutex_unlock(&dbg->sock_lock);
 		return FALSE;
 	}
 
-	pthread_mutex_unlock(&dbg->sock_lock);
 	return TRUE;
 }
 #endif
 
+static bool_t hdhomerun_debug_output_message(struct hdhomerun_debug_t *dbg, struct hdhomerun_debug_message_t *message)
+{
+	pthread_mutex_lock(&dbg->send_lock);
+
+	if (!dbg->enabled) {
+		pthread_mutex_unlock(&dbg->send_lock);
+		return TRUE;
+	}
+
+	bool_t ret;
+	if (dbg->file_name) {
+		ret = hdhomerun_debug_output_message_file(dbg, message);
+	} else {
+		ret = hdhomerun_debug_output_message_sock(dbg, message);
+	}
+
+	pthread_mutex_unlock(&dbg->send_lock);
+	return ret;
+}
+
 static void hdhomerun_debug_pop_and_free_message(struct hdhomerun_debug_t *dbg)
 {
 	pthread_mutex_lock(&dbg->queue_lock);
