@@ -34,6 +34,8 @@ struct hdhomerun_debug_t;
 extern struct hdhomerun_debug_t *hdhomerun_debug_create(void);
 extern void hdhomerun_debug_destroy(struct hdhomerun_debug_t *dbg);
 
+extern void hdhomerun_debug_set_prefix(struct hdhomerun_debug_t *dbg, const char *prefix);
+extern void hdhomerun_debug_set_filename(struct hdhomerun_debug_t *dbg, const char *filename);
 extern void hdhomerun_debug_enable(struct hdhomerun_debug_t *dbg);
 extern void hdhomerun_debug_disable(struct hdhomerun_debug_t *dbg);
 extern bool_t hdhomerun_debug_enabled(struct hdhomerun_debug_t *dbg);
