@@ -23,10 +23,13 @@
 
 #if defined(__CYGWIN__) || defined(__WINDOWS__)
 #include <windows.h>
-#include <iptypes.h>
 #include <iphlpapi.h>
-#else
+#define USE_IPHLPAPI 1
+#endif
+
+#if defined(__linux__) || defined(__APPLE__) || defined(BSD)
 #include <ifaddrs.h>
+#define USE_IFADDRS 1
 #endif
 
 #define HDHOMERUN_DISOCVER_MAX_SOCK_COUNT 16
@@ -39,6 +42,8 @@ struct hdhomerun_discover_sock_t {
 struct hdhomerun_discover_t {
 	struct hdhomerun_discover_sock_t socks[HDHOMERUN_DISOCVER_MAX_SOCK_COUNT];
 	unsigned int sock_count;
+	struct hdhomerun_pkt_t tx_pkt;
+	struct hdhomerun_pkt_t rx_pkt;
 };
 
 static void hdhomerun_discover_sock_create(struct hdhomerun_discover_t *ds, uint32_t local_ip, uint32_t mask)
@@ -78,8 +83,7 @@ static void hdhomerun_discover_sock_create(struct hdhomerun_discover_t *ds, uint
 	dss->broadcast_ip = local_ip | ~mask;
 }
 
-#if defined(__CYGWIN__) || defined(__WINDOWS__)
-
+#if defined(USE_IPHLPAPI)
 static void hdhomerun_discover_sock_detect(struct hdhomerun_discover_t *ds)
 {
 	PIP_ADAPTER_INFO pAdapterInfo = (IP_ADAPTER_INFO *)malloc(sizeof(IP_ADAPTER_INFO));
@@ -120,9 +124,9 @@ static void hdhomerun_discover_sock_detect(struct hdhomerun_discover_t *ds)
 
 	free(pAdapterInfo);
 }
+#endif
 
-#else
-
+#if defined(USE_IFADDRS)
 static void hdhomerun_discover_sock_detect(struct hdhomerun_discover_t *ds)
 {
 	struct ifaddrs *ifap;
@@ -152,7 +156,14 @@ static void hdhomerun_discover_sock_detect(struct hdhomerun_discover_t *ds)
 
 	freeifaddrs(ifap);
 }
+#endif
 
+
+#if !defined(USE_IPHLPAPI) && !defined(USE_IFADDRS)
+static void hdhomerun_discover_sock_detect(struct hdhomerun_discover_t *ds)
+{
+	/* do nothing */
+}
 #endif
 
 static struct hdhomerun_discover_t *hdhomerun_discover_create(void)
@@ -187,15 +198,22 @@ static void hdhomerun_discover_destroy(struct hdhomerun_discover_t *ds)
 	free(ds);
 }
 
-static bool_t hdhomerun_discover_send_internal(struct hdhomerun_discover_sock_t *dss, uint32_t target_ip, uint32_t device_type, uint32_t device_id)
+static bool_t hdhomerun_discover_send_internal(struct hdhomerun_discover_t *ds, struct hdhomerun_discover_sock_t *dss, uint32_t target_ip, uint32_t device_type, uint32_t device_id)
 {
 	if (target_ip == 0) {
 		target_ip = dss->broadcast_ip;
 	}
 
-	uint8_t buffer[1024];
-	uint8_t *ptr = buffer;
-	hdhomerun_write_discover_request(&ptr, device_type, device_id);
+	struct hdhomerun_pkt_t *tx_pkt = &ds->tx_pkt;
+	hdhomerun_pkt_reset(tx_pkt);
+
+	hdhomerun_pkt_write_u8(tx_pkt, HDHOMERUN_TAG_DEVICE_TYPE);
+	hdhomerun_pkt_write_var_length(tx_pkt, 4);
+	hdhomerun_pkt_write_u32(tx_pkt, device_type);
+	hdhomerun_pkt_write_u8(tx_pkt, HDHOMERUN_TAG_DEVICE_ID);
+	hdhomerun_pkt_write_var_length(tx_pkt, 4);
+	hdhomerun_pkt_write_u32(tx_pkt, device_id);
+	hdhomerun_pkt_seal_frame(tx_pkt, HDHOMERUN_TYPE_DISCOVER_REQ);
 
 	struct sockaddr_in sock_addr;
 	memset(&sock_addr, 0, sizeof(sock_addr));
@@ -203,8 +221,8 @@ static bool_t hdhomerun_discover_send_internal(struct hdhomerun_discover_sock_t
 	sock_addr.sin_addr.s_addr = htonl(target_ip);
 	sock_addr.sin_port = htons(HDHOMERUN_DISCOVER_UDP_PORT);
 
-	int length = (int)(ptr - buffer);
-	if (sendto(dss->sock, (char *)buffer, (int)length, 0, (struct sockaddr *)&sock_addr, sizeof(sock_addr)) != length) {
+	int length = (int)(tx_pkt->end - tx_pkt->start);
+	if (sendto(dss->sock, (char *)tx_pkt->start, length, 0, (struct sockaddr *)&sock_addr, sizeof(sock_addr)) != length) {
 		return FALSE;
 	}
 
@@ -218,35 +236,32 @@ static bool_t hdhomerun_discover_send(struct hdhomerun_discover_t *ds, uint32_t
 	unsigned int i;
 	for (i = 0; i < ds->sock_count; i++) {
 		struct hdhomerun_discover_sock_t *dss = &ds->socks[i];
-		result |= hdhomerun_discover_send_internal(dss, target_ip, device_type, device_id);
+		result |= hdhomerun_discover_send_internal(ds, dss, target_ip, device_type, device_id);
 	}
 
 	return result;
 }
 
-static int hdhomerun_discover_recv_internal(struct hdhomerun_discover_sock_t *dss, struct hdhomerun_discover_device_t *result)
+static int hdhomerun_discover_recv_internal(struct hdhomerun_discover_t *ds, struct hdhomerun_discover_sock_t *dss, struct hdhomerun_discover_device_t *result)
 {
-	uint8_t buffer[1024];
+	struct hdhomerun_pkt_t *rx_pkt = &ds->rx_pkt;
+	hdhomerun_pkt_reset(rx_pkt);
+
 	struct sockaddr_in sock_addr;
 	memset(&sock_addr, 0, sizeof(sock_addr));
 	socklen_t sockaddr_size = sizeof(sock_addr);
-	int rx_length = recvfrom(dss->sock, (char *)buffer, sizeof(buffer), 0, (struct sockaddr *)&sock_addr, &sockaddr_size);
+
+	int rx_length = recvfrom(dss->sock, (char *)rx_pkt->end, (int)(rx_pkt->limit - rx_pkt->end), 0, (struct sockaddr *)&sock_addr, &sockaddr_size);
 	if (rx_length <= 0) {
 		/* Don't return error - windows machine on VPN can sometimes cause a sock error here but otherwise works. */
 		return 0;
 	}
-	if (rx_length < HDHOMERUN_MIN_PEEK_LENGTH) {
+	rx_pkt->end += rx_length;
+
+	uint16_t type;
+	if (hdhomerun_pkt_open_frame(rx_pkt, &type) <= 0) {
 		return 0;
 	}
-
-	size_t length = hdhomerun_peek_packet_length(buffer);
-	if (length > (size_t)rx_length) {
-		return 0;
-	}
-
-	uint8_t *ptr = buffer;
-	uint8_t *end = buffer + length;
-	int type = hdhomerun_process_packet(&ptr, &end);
 	if (type != HDHOMERUN_TYPE_DISCOVER_RPY) {
 		return 0;
 	}
@@ -254,11 +269,12 @@ static int hdhomerun_discover_recv_internal(struct hdhomerun_discover_sock_t *ds
 	result->ip_addr = ntohl(sock_addr.sin_addr.s_addr);
 	result->device_type = 0;
 	result->device_id = 0;
+
 	while (1) {
 		uint8_t tag;
 		size_t len;
-		uint8_t *value;
-		if (hdhomerun_read_tlv(&ptr, end, &tag, &len, &value) < 0) {
+		uint8_t *next = hdhomerun_pkt_read_tlv(rx_pkt, &tag, &len);
+		if (!next) {
 			break;
 		}
 
@@ -267,17 +283,21 @@ static int hdhomerun_discover_recv_internal(struct hdhomerun_discover_sock_t *ds
 			if (len != 4) {
 				break;
 			}
-			result->device_type = hdhomerun_read_u32(&value);
+			result->device_type = hdhomerun_pkt_read_u32(rx_pkt);
 			break;
+
 		case HDHOMERUN_TAG_DEVICE_ID:
 			if (len != 4) {
 				break;
 			}
-			result->device_id = hdhomerun_read_u32(&value);
+			result->device_id = hdhomerun_pkt_read_u32(rx_pkt);
 			break;
+
 		default:
 			break;
 		}
+
+		rx_pkt->pos = next;
 	}
 
 	return 1;
@@ -312,7 +332,7 @@ static int hdhomerun_discover_recv(struct hdhomerun_discover_t *ds, struct hdhom
 			continue;
 		}
 
-		if (hdhomerun_discover_recv_internal(dss, result) <= 0) {
+		if (hdhomerun_discover_recv_internal(ds, dss, result) <= 0) {
 			continue;
 		}
 
